@@ -1,6 +1,6 @@
 /*
   The oSIP library implements the Session Initiation Protocol (SIP -rfc3261-)
-  Copyright (C) 2001,2002,2003,2004,2005,2006,2007 Aymeric MOIZARD jack@atosc.org
+  Copyright (C) 2001-2012 Aymeric MOIZARD amoizard@antisip.com
   
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
@@ -20,7 +20,7 @@
 #ifndef _FIFO_H_
 #define _FIFO_H_
 
-#ifdef OSIP_MT
+#ifndef OSIP_MONOTHREAD
 #include <osip2/osip_mt.h>
 #endif
 #include <osipparser2/osip_list.h>
@@ -46,8 +46,7 @@ extern "C" {
 
 #ifndef DOXYGEN
 
-#define MAX_LEN 1000
-	typedef enum { osip_ok, osip_full, osip_empty } osip_fifo_state;
+  typedef enum { osip_ok, osip_empty } osip_fifo_state;
 
 #endif
 
@@ -55,22 +54,21 @@ extern "C" {
  * Structure for referencing a fifo.
  * @var osip_fifo_t
  */
-	typedef struct osip_fifo osip_fifo_t;
+  typedef struct osip_fifo osip_fifo_t;
 
 /**
  * Structure for referencing a fifo.
  * @struct osip_fifo
  */
-	struct osip_fifo {
-#ifdef OSIP_MT
-		struct osip_mutex *qislocked;
-								   /**@internal */
-		struct osip_sem *qisempty; /**@internal */
+  struct osip_fifo {
+#ifndef OSIP_MONOTHREAD
+    struct osip_mutex *qislocked;          /**< mutex for fifo */
+    struct osip_sem *qisempty;             /**< semaphore for fifo */
 #endif
-		osip_list_t queue;		   /**< list of nodes containing elements */
-		int nb_elt;				   /**< nb of elements */
-		osip_fifo_state state;		 /**@internal state of the fifo */
-	};
+    osip_list_t queue;                     /**< list of nodes containing elements */
+    int nb_elt;                            /**< nb of elements */
+    osip_fifo_state state;                 /**< state of the fifo */
+  };
 
 /**
  * Initialise a osip_fifo_t element.
@@ -81,41 +79,41 @@ extern "C" {
  * if you want to use osip_fifo_free().
  * @param ff The element to initialise.
  */
-	void osip_fifo_init(osip_fifo_t * ff);
+  void osip_fifo_init (osip_fifo_t * ff);
 /**
  * Free a fifo element.
  * @param ff The element to work on.
  */
-	void osip_fifo_free(osip_fifo_t * ff);
+  void osip_fifo_free (osip_fifo_t * ff);
 /**
  * Insert an element in a fifo (at the beginning).
  * @param ff The element to work on.
  * @param element The pointer on the element to insert.
  */
-	int osip_fifo_insert(osip_fifo_t * ff, void *element);
+  int osip_fifo_insert (osip_fifo_t * ff, void *element);
 /**
  * Add an element in a fifo.
  * @param ff The element to work on.
  * @param element The pointer on the element to add.
  */
-	int osip_fifo_add(osip_fifo_t * ff, void *element);
+  int osip_fifo_add (osip_fifo_t * ff, void *element);
 /**
  * Get the number of element in a fifo.
  * @param ff The element to work on.
  */
-	int osip_fifo_size(osip_fifo_t * ff);
-#ifdef OSIP_MT
+  int osip_fifo_size (osip_fifo_t * ff);
+#ifndef OSIP_MONOTHREAD
 /**
  * Get an element from a fifo or block until one is added.
  * @param ff The element to work on.
  */
-	void *osip_fifo_get(osip_fifo_t * ff);
+  void *osip_fifo_get (osip_fifo_t * ff);
 #endif
 /**
  * Try to get an element from a fifo, but do not block if there is no element.
  * @param ff The element to work on.
  */
-	void *osip_fifo_tryget(osip_fifo_t * ff);
+  void *osip_fifo_tryget (osip_fifo_t * ff);
 
 
 /** @} */
