@@ -1,6 +1,6 @@
 /*
-  The oSIP library implements the Session Initiation Protocol (SIP -rfc2543-)
-  Copyright (C) 2001  Aymeric MOIZARD jack@atosc.org
+  The oSIP library implements the Session Initiation Protocol (SIP -rfc3261-)
+  Copyright (C) 2001-2015 Aymeric MOIZARD amoizard@antisip.com
   
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
@@ -36,7 +36,7 @@
 
 /**
  * Structure for Allow headers.
- * @defvar osip_allow_t
+ * @var osip_allow_t
  */
   typedef osip_content_length_t osip_allow_t;
 
@@ -61,18 +61,18 @@ extern "C"
  * @param header The element to work on.
  * @param dest A pointer on the new allocated string.
  */
-#define osip_allow_to_str(header, dest)  osip_content_length_to_str(header, dest)
+#define osip_allow_to_str  osip_content_length_to_str
 /**
  * Free a Allow element.
  * @param header The element to work on.
  */
-#define osip_allow_free(header)      osip_content_length_free(header)
+#define osip_allow_free      osip_content_length_free
 /**
  * Clone a Allow element.
  * @param header The element to work on.
  * @param dest A pointer on the copy of the element.
  */
-#define osip_allow_clone(header, dest)  osip_content_length_clone(header, dest)
+#define osip_allow_clone  osip_content_length_clone
 
 
 #ifdef __cplusplus
