@@ -1,6 +1,6 @@
 /*
   The oSIP library implements the Session Initiation Protocol (SIP -rfc3261-)
-  Copyright (C) 2001,2002,2003,2004,2005,2006,2007 Aymeric MOIZARD jack@atosc.org
+  Copyright (C) 2001-2015 Aymeric MOIZARD amoizard@antisip.com
   
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
@@ -36,8 +36,7 @@
  */
 
 #ifdef __cplusplus
-extern "C"
-{
+extern "C" {
 #endif
 
 /**
@@ -49,8 +48,7 @@ extern "C"
  * Fix the via header for INCOMING requests only.
  * a copy of ip_addr is done.
  */
-  int osip_message_fix_last_via_header (osip_message_t * request,
-                                        const char *ip_addr, int port);
+  int osip_message_fix_last_via_header (osip_message_t * request, const char *ip_addr, int port);
 
 
 /**
@@ -94,8 +92,7 @@ extern "C"
  * @param dest A pointer on the header found.
  */
 #ifndef MINISIZE
-  int osip_message_get_accept (const osip_message_t * sip, int pos,
-                               osip_accept_t ** dest);
+  int osip_message_get_accept (const osip_message_t * sip, int pos, osip_accept_t ** dest);
 #else
 #define osip_message_get_accept(sip,pos,dest)          osip_message_header_get_byname(( osip_message_t *)sip,(const char *)"accept",pos,(osip_header_t **)dest)
 #endif
@@ -116,8 +113,7 @@ extern "C"
  * @param dest A pointer on the header found.
  */
 #ifndef MINISIZE
-  int osip_message_get_accept_encoding (const osip_message_t * sip, int pos,
-                                        osip_accept_encoding_t ** dest);
+  int osip_message_get_accept_encoding (const osip_message_t * sip, int pos, osip_accept_encoding_t ** dest);
 #else
 #define osip_message_get_accept_encoding(sip,pos,dest)          osip_message_header_get_byname(( osip_message_t *)sip,(const char *)"accept-encoding",pos,(osip_header_t **)dest)
 #endif
@@ -138,8 +134,7 @@ extern "C"
  * @param dest A pointer on the header found.
  */
 #ifndef MINISIZE
-  int osip_message_get_accept_language (const osip_message_t * sip, int pos,
-                                        osip_accept_language_t ** dest);
+  int osip_message_get_accept_language (const osip_message_t * sip, int pos, osip_accept_language_t ** dest);
 #else
 #define osip_message_get_accept_language(sip,pos,dest)          osip_message_header_get_byname(( osip_message_t *)sip,(const char *)"accept-language",pos,(osip_header_t **)dest)
 #endif
@@ -160,8 +155,7 @@ extern "C"
  * @param dest A pointer on the header found.
  */
 #ifndef MINISIZE
-  int osip_message_get_alert_info (const osip_message_t * sip, int pos,
-                                   osip_alert_info_t ** dest);
+  int osip_message_get_alert_info (const osip_message_t * sip, int pos, osip_alert_info_t ** dest);
 #else
 #define osip_message_get_alert_info(sip,pos,dest)          osip_message_header_get_byname(( osip_message_t *)sip,(const char *)"alert-info",pos,(osip_header_t **)dest)
 #endif
@@ -182,8 +176,7 @@ extern "C"
  * @param dest A pointer on the header found.
  */
 #ifndef MINISIZE
-  int osip_message_get_allow (const osip_message_t * sip, int pos,
-                              osip_allow_t ** dest);
+  int osip_message_get_allow (const osip_message_t * sip, int pos, osip_allow_t ** dest);
 #else
 #define osip_message_get_allow(sip,pos,dest)          osip_message_header_get_byname(( osip_message_t *)sip,(const char *)"allow",pos,(osip_header_t **)dest)
 #endif
@@ -193,8 +186,7 @@ extern "C"
  * @param hvalue The string describing the element.
  */
 #ifndef MINISIZE
-  int osip_message_set_authentication_info (osip_message_t * sip,
-                                            const char *hvalue);
+  int osip_message_set_authentication_info (osip_message_t * sip, const char *hvalue);
 #else
 #define osip_message_set_authentication_info(sip,value)            osip_message_set_header((osip_message_t *)sip,(const char *)"Authentication-Info",value)
 #endif
@@ -205,9 +197,7 @@ extern "C"
  * @param dest A pointer on the header found.
  */
 #ifndef MINISIZE
-  int osip_message_get_authentication_info (const osip_message_t * sip,
-                                            int pos,
-                                            osip_authentication_info_t ** dest);
+  int osip_message_get_authentication_info (const osip_message_t * sip, int pos, osip_authentication_info_t ** dest);
 #else
 #define osip_message_get_authentication_info(sip,pos,dest)          osip_message_header_get_byname(( osip_message_t *)sip,(const char *)"authentication-info",pos,(osip_header_t **)dest)
 #endif
@@ -224,8 +214,7 @@ extern "C"
  * @param dest A pointer on the header found.
  */
 #ifndef MINISIZE
-  int osip_message_get_authorization (const osip_message_t * sip, int pos,
-                                      osip_authorization_t ** dest);
+  int osip_message_get_authorization (const osip_message_t * sip, int pos, osip_authorization_t ** dest);
 #else
 #define osip_message_get_authorization(sip, pos, dest) osip_message_get_knownheaderlist((&(sip)->authorizations), pos, (void **)(dest))
 #endif
@@ -249,23 +238,14 @@ extern "C"
  * @param sip The element to work on.
  * @param hvalue The string describing the element.
  */
-#ifndef MINISIZE
   int osip_message_set_call_info (osip_message_t * sip, const char *hvalue);
-#else
-#define osip_message_set_call_info(sip,value)            osip_message_set_header((osip_message_t *)sip,(const char *)"Call-Info",value)
-#endif
 /**
  * Get one Call-info header.
  * @param sip The element to work on.
  * @param pos The index of the element to get.
  * @param dest A pointer on the header found.
  */
-#ifndef MINISIZE
-  int osip_message_get_call_info (const osip_message_t * sip, int pos,
-                                  osip_call_info_t ** dest);
-#else
-#define osip_message_get_call_info(sip,pos,dest)          osip_message_header_get_byname(( osip_message_t *)sip,(const char *)"call-info",pos,(osip_header_t **)dest)
-#endif
+  int osip_message_get_call_info (const osip_message_t * sip, int pos, osip_call_info_t ** dest);
 /**
  * Set the Contact header.
  * @param sip The element to work on.
@@ -279,10 +259,9 @@ extern "C"
  * @param dest A pointer on the header found.
  */
 #ifndef MINISIZE
-  int osip_message_get_contact (const osip_message_t * sip, int pos,
-                                osip_contact_t ** dest);
+  int osip_message_get_contact (const osip_message_t * sip, int pos, osip_contact_t ** dest);
 #else
-#define osip_message_get_contact(sip, pos, dest) osip_message_get_knownheaderlist((&(sip)->contacts), pos, (void **)(dest)) 
+#define osip_message_get_contact(sip, pos, dest) osip_message_get_knownheaderlist((&(sip)->contacts), pos, (void **)(dest))
 #endif
 /**
  * Set the Content-encoding header.
@@ -301,8 +280,7 @@ extern "C"
  * @param dest A pointer on the header found.
  */
 #ifndef MINISIZE
-  int osip_message_get_content_encoding (const osip_message_t * sip, int pos,
-                                         osip_content_encoding_t ** dest);
+  int osip_message_get_content_encoding (const osip_message_t * sip, int pos, osip_content_encoding_t ** dest);
 #else
 #define osip_message_get_content_encoding(sip,pos,dest)          osip_message_header_get_byname(( osip_message_t *)sip,(const char *)"content-encoding",pos,(osip_header_t **)dest)
 #endif
@@ -317,8 +295,7 @@ extern "C"
  * @param sip The element to work on.
  */
 #ifndef MINISIZE
-  osip_content_length_t *osip_message_get_content_length (const osip_message_t
-                                                          * sip);
+  osip_content_length_t *osip_message_get_content_length (const osip_message_t * sip);
 #else
 #define osip_message_get_content_length(sip)   ((sip)->content_length)
 #endif
@@ -369,8 +346,7 @@ extern "C"
  * @param dest A pointer on the header found.
  */
 #ifndef MINISIZE
-  int osip_message_get_error_info (const osip_message_t * sip, int pos,
-                                   osip_error_info_t ** dest);
+  int osip_message_get_error_info (const osip_message_t * sip, int pos, osip_error_info_t ** dest);
 #else
 #define osip_message_get_error_info(sip,pos,dest)          osip_message_header_get_byname(( osip_message_t *)sip,(const char *)"error-info",pos,(osip_header_t **)dest)
 #endif
@@ -409,8 +385,7 @@ extern "C"
  * @param sip The element to work on.
  * @param hvalue The string describing the element.
  */
-  int osip_message_set_proxy_authenticate (osip_message_t * sip,
-                                           const char *hvalue);
+  int osip_message_set_proxy_authenticate (osip_message_t * sip, const char *hvalue);
 /**
  * Get the Proxy-authenticate header.
  * @param sip The element to work on.
@@ -418,19 +393,16 @@ extern "C"
  * @param dest A pointer on the header found.
  */
 #ifndef MINISIZE
-  int osip_message_get_proxy_authenticate (const osip_message_t * sip,
-                                           int pos,
-                                           osip_proxy_authenticate_t ** dest);
+  int osip_message_get_proxy_authenticate (const osip_message_t * sip, int pos, osip_proxy_authenticate_t ** dest);
 #else
-#define osip_message_get_proxy_authenticate(sip, pos, dest) osip_message_get_knownheaderlist((&(sip)->proxy_authenticates), pos, (void **)(dest)) 
+#define osip_message_get_proxy_authenticate(sip, pos, dest) osip_message_get_knownheaderlist((&(sip)->proxy_authenticates), pos, (void **)(dest))
 #endif
 /**
  * Set the Proxy-authorization header.
  * @param sip The element to work on.
  * @param hvalue The string describing the element.
  */
-  int osip_message_set_proxy_authorization (osip_message_t * sip,
-                                            const char *hvalue);
+  int osip_message_set_proxy_authorization (osip_message_t * sip, const char *hvalue);
 /**
  * Get one Proxy-authorization header.
  * @param sip The element to work on.
@@ -438,11 +410,9 @@ extern "C"
  * @param dest A pointer on the header found.
  */
 #ifndef MINISIZE
-  int osip_message_get_proxy_authorization (const osip_message_t * sip,
-                                            int pos,
-                                            osip_proxy_authorization_t ** dest);
+  int osip_message_get_proxy_authorization (const osip_message_t * sip, int pos, osip_proxy_authorization_t ** dest);
 #else
-#define osip_message_get_proxy_authorization(sip, pos, dest) osip_message_get_knownheaderlist((&(sip)->proxy_authorizations), pos, (void **)(dest)) 
+#define osip_message_get_proxy_authorization(sip, pos, dest) osip_message_get_knownheaderlist((&(sip)->proxy_authorizations), pos, (void **)(dest))
 #endif
 /**
  * Set the Proxy-authentication-info header.
@@ -450,8 +420,7 @@ extern "C"
  * @param hvalue The string describing the element.
  */
 #ifndef MINISIZE
-  int osip_message_set_proxy_authentication_info (osip_message_t * sip,
-                                                  const char *hvalue);
+  int osip_message_set_proxy_authentication_info (osip_message_t * sip, const char *hvalue);
 #else
 #define osip_message_set_proxy_authentication_info(sip,value)            osip_message_set_header((osip_message_t *)sip,(const char *)"Proxy-Authentication-Info",value)
 #endif
@@ -462,10 +431,7 @@ extern "C"
  * @param dest A pointer on the header found.
  */
 #ifndef MINISIZE
-  int osip_message_get_proxy_authentication_info (const osip_message_t * sip,
-                                                  int pos,
-                                                  osip_proxy_authentication_info_t
-                                                  ** dest);
+  int osip_message_get_proxy_authentication_info (const osip_message_t * sip, int pos, osip_proxy_authentication_info_t ** dest);
 #else
 #define osip_message_get_proxy_authentication_info(sip,pos,dest)          osip_message_header_get_byname(( osip_message_t *)sip,(const char *)"proxy-authentication-info",pos,(osip_header_t **)dest)
 #endif
@@ -482,10 +448,9 @@ extern "C"
  * @param dest A pointer on the header found.
  */
 #ifndef MINISIZE
-  int osip_message_get_record_route (const osip_message_t * sip, int pos,
-                                     osip_record_route_t ** dest);
+  int osip_message_get_record_route (const osip_message_t * sip, int pos, osip_record_route_t ** dest);
 #else
-#define osip_message_get_record_route(sip, pos, dest) osip_message_get_knownheaderlist((&(sip)->record_routes), pos, (void **)(dest)) 
+#define osip_message_get_record_route(sip, pos, dest) osip_message_get_knownheaderlist((&(sip)->record_routes), pos, (void **)(dest))
 #endif
 /**
  * Set the Route header.
@@ -500,10 +465,9 @@ extern "C"
  * @param dest A pointer on the header found.
  */
 #ifndef MINISIZE
-  int osip_message_get_route (const osip_message_t * sip, int pos,
-                              osip_route_t ** dest);
+  int osip_message_get_route (const osip_message_t * sip, int pos, osip_route_t ** dest);
 #else
-#define osip_message_get_route(sip, pos, dest) osip_message_get_knownheaderlist((&(sip)->routes), pos, (void **)(dest)) 
+#define osip_message_get_route(sip, pos, dest) osip_message_get_knownheaderlist((&(sip)->routes), pos, (void **)(dest))
 #endif
 /**
  * Set the To header.
@@ -539,10 +503,9 @@ extern "C"
  * @param dest A pointer on the header found.
  */
 #ifndef MINISIZE
-  int osip_message_get_via (const osip_message_t * sip, int pos,
-                            osip_via_t ** dest);
+  int osip_message_get_via (const osip_message_t * sip, int pos, osip_via_t ** dest);
 #else
-#define osip_message_get_via(sip, pos, dest) osip_message_get_knownheaderlist((&(sip)->vias), pos, (void **)(dest)) 
+#define osip_message_get_via(sip, pos, dest) osip_message_get_knownheaderlist((&(sip)->vias), pos, (void **)(dest))
 #endif
 /**
  * Set the Www-authenticate header.
@@ -557,10 +520,9 @@ extern "C"
  * @param dest A pointer on the header found.
  */
 #ifndef MINISIZE
-  int osip_message_get_www_authenticate (const osip_message_t * sip, int pos,
-                                         osip_www_authenticate_t ** dest);
+  int osip_message_get_www_authenticate (const osip_message_t * sip, int pos, osip_www_authenticate_t ** dest);
 #else
-#define osip_message_get_www_authenticate(sip, pos, dest) osip_message_get_knownheaderlist((&(sip)->www_authenticates), pos, (void **)(dest)) 
+#define osip_message_get_www_authenticate(sip, pos, dest) osip_message_get_knownheaderlist((&(sip)->www_authenticates), pos, (void **)(dest))
 #endif
 #ifndef DOXYGEN
 /**
@@ -569,8 +531,7 @@ extern "C"
  * @param hname The token name. NAME MUST BE DYNAMICLY ALLOCATED
  * @param hvalue The token value. VALUE MUST BE DYNAMICLY ALLOCATED
  */
-  int osip_message_set_multiple_header (osip_message_t * sip, char *hname,
-                                        char *hvalue);
+  int osip_message_set_multiple_header (osip_message_t * sip, char *hname, char *hvalue);
 #endif
 /**
  * Allocate and Add an "unknown" header (not defined in oSIP).
@@ -578,16 +539,14 @@ extern "C"
  * @param hname The token name.
  * @param hvalue The token value.
  */
-  int osip_message_set_header (osip_message_t * sip, const char *hname,
-                               const char *hvalue);
+  int osip_message_set_header (osip_message_t * sip, const char *hname, const char *hvalue);
 /**
  * Allocate and Add/Replace an "unknown" header (not defined in oSIP).
  * @param sip The element to work on.
  * @param hname The token name.
  * @param hvalue The token value.
  */
-  int osip_message_replace_header (osip_message_t * sip, const char *hname,
-				   const char *hvalue);
+  int osip_message_replace_header (osip_message_t * sip, const char *hname, const char *hvalue);
 #ifndef MINISIZE
 /**
  * Allocate and Add an "unknown" header (not defined in oSIP).
@@ -596,8 +555,7 @@ extern "C"
  * @param hname The token name.
  * @param hvalue The token value.
  */
-  int osip_message_set_topheader (osip_message_t * sip, const char *hname,
-                                  const char *hvalue);
+  int osip_message_set_topheader (osip_message_t * sip, const char *hname, const char *hvalue);
 #endif
 /**
  * Find an "unknown" header. (not defined in oSIP)
@@ -606,9 +564,7 @@ extern "C"
  * @param pos The index where to start searching for the header.
  * @param dest A pointer to the header found.
  */
-  int osip_message_header_get_byname (const osip_message_t * sip,
-                                      const char *hname, int pos,
-                                      osip_header_t ** dest);
+  int osip_message_header_get_byname (const osip_message_t * sip, const char *hname, int pos, osip_header_t ** dest);
 /**
  * Get one "unknown" header.
  * @param sip The element to work on.
@@ -616,10 +572,9 @@ extern "C"
  * @param dest A pointer on the header found.
  */
 #ifndef MINISZE
-  int osip_message_get_header (const osip_message_t * sip, int pos,
-                               osip_header_t ** dest);
+  int osip_message_get_header (const osip_message_t * sip, int pos, osip_header_t ** dest);
 #else
-#define osip_message_get_header(sip, pos, dest) osip_message_get_knownheaderlist((&(sip)->headers), pos, (void **)(dest)) 
+#define osip_message_get_header(sip, pos, dest) osip_message_get_knownheaderlist((&(sip)->headers), pos, (void **)(dest))
 #endif
 /**
  * Set the Body of the SIP message.
@@ -634,8 +589,7 @@ extern "C"
  * @param buf the buffer containing the body.
  * @param length The length of the buffer.
  */
-  int osip_message_set_body_mime (osip_message_t * sip, const char *buf,
-                                  size_t length);
+  int osip_message_set_body_mime (osip_message_t * sip, const char *buf, size_t length);
 /**
  * Get one body header.
  * @param sip The element to work on.
@@ -643,10 +597,9 @@ extern "C"
  * @param dest A pointer on the body found.
  */
 #ifndef MINISIZE
-  int osip_message_get_body (const osip_message_t * sip, int pos,
-                             osip_body_t ** dest);
+  int osip_message_get_body (const osip_message_t * sip, int pos, osip_body_t ** dest);
 #else
-#define osip_message_get_body(sip, pos, dest) osip_message_get_knownheaderlist((&(sip)->bodies), pos, (void **)(dest)) 
+#define osip_message_get_body(sip, pos, dest) osip_message_get_knownheaderlist((&(sip)->bodies), pos, (void **)(dest))
 #endif
 
 
@@ -916,5 +869,4 @@ extern "C"
 #ifdef __cplusplus
 }
 #endif
-
 #endif
