@@ -230,6 +230,7 @@ osip_compensatetime ()
 
   _osip_gettimeofday_realtime (&now_real, NULL);
   osip_gettimeofday (&now_monotonic, NULL);
+  now_monotonic.tv_sec -= offset.tv_sec;
 
   if (now_real.tv_sec == 0)
     return;                     /* no compensation */
@@ -241,6 +242,7 @@ osip_compensatetime ()
   if (last_now_monotonic.tv_sec == 0) {
     _osip_gettimeofday_realtime (&last_now_real, NULL);
     osip_gettimeofday (&last_now_monotonic, NULL);
+    last_now_monotonic.tv_sec -= offset.tv_sec;
 
     return;
   }
@@ -248,10 +250,6 @@ osip_compensatetime ()
   diff_monotonic.tv_sec = now_monotonic.tv_sec - last_now_monotonic.tv_sec;
   diff_real.tv_sec = now_real.tv_sec - last_now_real.tv_sec;
 
-  /* reset for later use */
-  _osip_gettimeofday_realtime (&last_now_real, NULL);
-  osip_gettimeofday (&last_now_monotonic, NULL);
-
   if (diff_real.tv_sec < 5)
     return;                     /* skip any "back in time" operation or small interval */
   if (diff_real.tv_sec > 3600)
@@ -261,6 +259,11 @@ osip_compensatetime ()
 
   OSIP_TRACE (osip_trace (__FILE__, __LINE__, OSIP_WARNING, NULL, "adjusting exosip monotonic time (%i)!\n", diff_real.tv_sec - diff_monotonic.tv_sec));
   offset.tv_sec += diff_real.tv_sec - diff_monotonic.tv_sec;
+
+  /* reset for later use */
+  _osip_gettimeofday_realtime (&last_now_real, NULL);
+  osip_gettimeofday (&last_now_monotonic, NULL);
+  last_now_monotonic.tv_sec -= offset.tv_sec;
 }
 
 time_t
@@ -268,6 +271,10 @@ osip_getsystemtime (time_t * t)
 {
   struct timeval now_monotonic;
 
+#ifdef ANDROID
+  osip_compensatetime ();
+#endif
+
   osip_gettimeofday (&now_monotonic, NULL);
   if (t != NULL) {
     *t = now_monotonic.tv_sec;
