@@ -1,6 +1,6 @@
 /*
   The oSIP library implements the Session Initiation Protocol (SIP -rfc3261-)
-  Copyright (C) 2001-2012 Aymeric MOIZARD amoizard@antisip.com
+  Copyright (C) 2001-2015 Aymeric MOIZARD amoizard@antisip.com
   
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
@@ -107,19 +107,10 @@ osip_accept_to_str (const osip_accept_t * accept, char **dest)
 
   tmp = tmp + strlen (tmp);
   {
-    int pos = 0;
-    osip_generic_param_t *u_param;
-
-#if 0
-    if (!osip_list_eol (&accept->gen_params, pos)) {    /* needed for cannonical form! (authentication issue of rfc2543) */
-      sprintf (tmp, " ");
-      tmp++;
-    }
-#endif
-    while (!osip_list_eol (&accept->gen_params, pos)) {
+    osip_list_iterator_t it;
+    osip_generic_param_t *u_param = (osip_generic_param_t*) osip_list_get_first(&accept->gen_params, &it);
+    while (u_param != OSIP_SUCCESS) {
       size_t tmp_len;
-
-      u_param = (osip_generic_param_t *) osip_list_get (&accept->gen_params, pos);
       if (u_param->gvalue == NULL) {
         osip_free (buf);
         return OSIP_SYNTAXERROR;
@@ -133,7 +124,7 @@ osip_accept_to_str (const osip_accept_t * accept, char **dest)
       }
       snprintf (tmp, len - (tmp - buf), "; %s=%s", u_param->gname, u_param->gvalue);
       tmp = tmp + strlen (tmp);
-      pos++;
+      u_param = (osip_generic_param_t *) osip_list_get_next(&it);
     }
   }
   *dest = buf;
