@@ -1,6 +1,6 @@
 /*
   The oSIP library implements the Session Initiation Protocol (SIP -rfc3261-)
-  Copyright (C) 2001-2012 Aymeric MOIZARD amoizard@antisip.com
+  Copyright (C) 2001-2015 Aymeric MOIZARD amoizard@antisip.com
   
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
@@ -348,7 +348,6 @@ osip_body_to_str (const osip_body_t * body, char **dest, size_t * str_length)
   char *tmp_body;
   char *tmp;
   char *ptr;
-  int pos;
   int i;
   size_t length;
 
@@ -392,31 +391,39 @@ osip_body_to_str (const osip_body_t * body, char **dest, size_t * str_length)
     tmp_body = osip_strn_append (tmp_body, CRLF, 2);
   }
 
-  pos = 0;
-  while (!osip_list_eol (body->headers, pos)) {
-    osip_header_t *header;
-
-    header = (osip_header_t *) osip_list_get (body->headers, pos);
-    i = osip_header_to_str (header, &tmp);
-    if (i != 0) {
-      osip_free (ptr);
-      return i;
+  {
+    osip_list_iterator_t it;
+    osip_header_t *header = (osip_header_t *) osip_list_get_first(body->headers, &it);  
+    while (header != OSIP_SUCCESS) {
+      i = osip_header_to_str (header, &tmp);
+      if (i != 0) {
+	osip_free (ptr);
+	return i;
+      }
+      if (length < tmp_body - ptr + strlen (tmp) + 4) {
+	size_t len;
+	
+	len = tmp_body - ptr;
+	length = length + strlen (tmp) + 4;
+	ptr = osip_realloc (ptr, length);
+	tmp_body = ptr + len;
+      }
+      tmp_body = osip_str_append (tmp_body, tmp);
+      osip_free (tmp);
+      tmp_body = osip_strn_append (tmp_body, CRLF, 2);
+      header = (osip_header_t *) osip_list_get_next(&it);
     }
-    if (length < tmp_body - ptr + strlen (tmp) + 4) {
+  }
+  
+  if ((osip_list_size (body->headers) > 0) || (body->content_type != NULL)) {
+    if (length < tmp_body - ptr + 3) {
       size_t len;
 
       len = tmp_body - ptr;
-      length = length + strlen (tmp) + 4;
+      length = length + 3 + body->length; /* add body->length, to avoid calling realloc often */
       ptr = osip_realloc (ptr, length);
       tmp_body = ptr + len;
     }
-    tmp_body = osip_str_append (tmp_body, tmp);
-    osip_free (tmp);
-    tmp_body = osip_strn_append (tmp_body, CRLF, 2);
-    pos++;
-  }
-
-  if ((osip_list_size (body->headers) > 0) || (body->content_type != NULL)) {
     tmp_body = osip_strn_append (tmp_body, CRLF, 2);
   }
   if (length < tmp_body - ptr + body->length + 4) {
