@@ -1,6 +1,6 @@
 /*
   The oSIP library implements the Session Initiation Protocol (SIP -rfc3261-)
-  Copyright (C) 2001-2012 Aymeric MOIZARD amoizard@antisip.com
+  Copyright (C) 2001-2015 Aymeric MOIZARD amoizard@antisip.com
   
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
@@ -24,8 +24,6 @@
 #include <osipparser2/osip_parser.h>
 #include "parser.h"
 
-#ifndef MINISIZE
-
 int
 osip_message_set_call_info (osip_message_t * sip, const char *hvalue)
 {
@@ -133,11 +131,9 @@ osip_call_info_to_str (const osip_call_info_t * call_info, char **dest)
   sprintf (buf, "%s", call_info->element);
 
   {
-    int pos = 0;
-    osip_generic_param_t *u_param;
-
-    while (!osip_list_eol (&call_info->gen_params, pos)) {
-      u_param = (osip_generic_param_t *) osip_list_get (&call_info->gen_params, pos);
+    osip_list_iterator_t it;
+    osip_generic_param_t *u_param = (osip_generic_param_t*) osip_list_get_first(&call_info->gen_params, &it);
+    while (u_param != OSIP_SUCCESS) {
       if (u_param->gvalue == NULL)
         plen = strlen (u_param->gname) + 2;
       else
@@ -150,7 +146,7 @@ osip_call_info_to_str (const osip_call_info_t * call_info, char **dest)
         sprintf (tmp, ";%s", u_param->gname);
       else
         sprintf (tmp, ";%s=%s", u_param->gname, u_param->gvalue);
-      pos++;
+      u_param = (osip_generic_param_t *) osip_list_get_next(&it);
     }
   }
   *dest = buf;
@@ -217,4 +213,3 @@ osip_call_info_set_uri (osip_call_info_t * ae, char *uri)
   ae->element = uri;
 }
 
-#endif
