@@ -1,6 +1,6 @@
 /*
   The oSIP library implements the Session Initiation Protocol (SIP -rfc3261-)
-  Copyright (C) 2001-2012 Aymeric MOIZARD amoizard@antisip.com
+  Copyright (C) 2001-2015 Aymeric MOIZARD amoizard@antisip.com
   
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
@@ -24,7 +24,6 @@
 #include <osipparser2/osip_parser.h>
 #include "parser.h"
 
-#ifndef MINISIZE
 
 int
 osip_content_disposition_parse (osip_content_disposition_t * cd, const char *hvalue)
@@ -52,4 +51,3 @@ osip_content_disposition_parse (osip_content_disposition_t * cd, const char *hva
   return OSIP_SUCCESS;
 }
 
-#endif
