@@ -1,6 +1,6 @@
 /*
   The oSIP library implements the Session Initiation Protocol (SIP -rfc3261-)
-  Copyright (C) 2001-2012 Aymeric MOIZARD amoizard@antisip.com
+  Copyright (C) 2001-2015 Aymeric MOIZARD amoizard@antisip.com
   
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
@@ -164,19 +164,10 @@ osip_content_type_to_str (const osip_content_type_t * content_type, char **dest)
 
   tmp = tmp + strlen (tmp);
   {
-    int pos = 0;
-    osip_generic_param_t *u_param;
-
-#if 0
-    if (!osip_list_eol (content_type->gen_params, pos)) {       /* needed for cannonical form! (authentication issue of rfc2543) */
-      sprintf (tmp, " ");
-      tmp++;
-    }
-#endif
-    while (!osip_list_eol (&content_type->gen_params, pos)) {
+    osip_list_iterator_t it;
+    osip_generic_param_t *u_param = (osip_generic_param_t*) osip_list_get_first(&content_type->gen_params, &it);
+    while (u_param != OSIP_SUCCESS) {
       size_t tmp_len;
-
-      u_param = (osip_generic_param_t *) osip_list_get (&content_type->gen_params, pos);
       if (u_param->gvalue == NULL) {
         osip_free (buf);
         return OSIP_SYNTAXERROR;
@@ -190,7 +181,7 @@ osip_content_type_to_str (const osip_content_type_t * content_type, char **dest)
       }
       snprintf (tmp, len - (tmp - buf), "; %s=%s", u_param->gname, u_param->gvalue);
       tmp = tmp + strlen (tmp);
-      pos++;
+      u_param = (osip_generic_param_t *) osip_list_get_next(&it);
     }
   }
   *dest = buf;
@@ -235,19 +226,17 @@ osip_content_type_clone (const osip_content_type_t * ctt, osip_content_type_t **
     ct->subtype = osip_strdup (ctt->subtype);
 
   {
-    int pos = 0;
-    osip_generic_param_t *u_param;
     osip_generic_param_t *dest_param;
-
-    while (!osip_list_eol (&ctt->gen_params, pos)) {
-      u_param = (osip_generic_param_t *) osip_list_get (&ctt->gen_params, pos);
+    osip_list_iterator_t it;
+    osip_generic_param_t *u_param = (osip_generic_param_t*) osip_list_get_first(&ctt->gen_params, &it);
+    while (u_param != OSIP_SUCCESS) {
       i = osip_generic_param_clone (u_param, &dest_param);
       if (i != 0) {
         osip_content_type_free (ct);
         return i;
       }
       osip_list_add (&ct->gen_params, dest_param, -1);
-      pos++;
+      u_param = (osip_generic_param_t *) osip_list_get_next(&it);
     }
   }
   *dest = ct;
