@@ -1,6 +1,6 @@
 /*
   The oSIP library implements the Session Initiation Protocol (SIP -rfc3261-)
-  Copyright (C) 2001-2012 Aymeric MOIZARD amoizard@antisip.com
+  Copyright (C) 2001-2015 Aymeric MOIZARD amoizard@antisip.com
   
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
@@ -120,14 +120,11 @@ osip_record_route_to_str (const osip_record_route_t * record_route, char **dest)
   osip_free (url);
 
   {
-    int pos = 0;
-    osip_generic_param_t *u_param;
     size_t plen;
     char *tmp;
-
-    while (!osip_list_eol (&record_route->gen_params, pos)) {
-      u_param = (osip_generic_param_t *) osip_list_get (&record_route->gen_params, pos);
-
+    osip_list_iterator_t it;
+    osip_generic_param_t *u_param = (osip_generic_param_t*) osip_list_get_first(&record_route->gen_params, &it);
+    while (u_param != OSIP_SUCCESS) {
       if (u_param->gvalue == NULL)
         plen = strlen (u_param->gname) + 2;
       else
@@ -140,7 +137,7 @@ osip_record_route_to_str (const osip_record_route_t * record_route, char **dest)
         snprintf (tmp, len - (tmp - buf), ";%s", u_param->gname);
       else
         snprintf (tmp, len - (tmp - buf), ";%s=%s", u_param->gname, u_param->gvalue);
-      pos++;
+      u_param = (osip_generic_param_t *) osip_list_get_next(&it);
     }
   }
   *dest = buf;
