@@ -1,6 +1,6 @@
 /*
   The oSIP library implements the Session Initiation Protocol (SIP -rfc3261-)
-  Copyright (C) 2001-2012 Aymeric MOIZARD amoizard@antisip.com
+  Copyright (C) 2001-2015 Aymeric MOIZARD amoizard@antisip.com
   
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
@@ -22,6 +22,10 @@
 #include <osipparser2/osip_port.h>
 #include <osipparser2/osip_message.h>
 
+#define osip_is_alpha(in) (  \
+       (in >= 'a' && in <= 'z') || \
+       (in >= 'A' && in <= 'Z'))
+
 /* allocate a new url structure */
 /* OUTPUT: osip_uri_t *url | structure to save results.   */
 /* OUTPUT: err_t *err | structure to store error.   */
@@ -65,11 +69,11 @@ osip_uri_init (osip_uri_t ** url)
 /* this method search for the separator and   */
 /* return it only if it is located before the */
 /* second separator. */
-char *
+const char *
 next_separator (const char *ch, int separator_osip_to_find, int before_separator)
 {
-  char *ind;
-  char *tmp;
+  const char *ind;
+  const char *tmp;
 
   ind = strchr (ch, separator_osip_to_find);
   if (ind == NULL)
@@ -97,14 +101,15 @@ next_separator (const char *ch, int separator_osip_to_find, int before_separator
 int
 osip_uri_parse (osip_uri_t * url, const char *buf)
 {
-  char *username;
-  char *password;
-  char *host;
+  const char *username;
+  const char *password;
+  const char *host;
   const char *port;
   const char *params;
   const char *headers;
   const char *tmp;
-
+  int i;
+  
   /* basic tests */
   if (buf == NULL || buf[0] == '\0')
     return OSIP_BADPARAMETER;
@@ -115,6 +120,14 @@ osip_uri_parse (osip_uri_t * url, const char *buf)
 
   if (tmp - buf < 2)
     return OSIP_SYNTAXERROR;
+
+  i=0;
+  while (buf+i<tmp) {
+    if (!osip_is_alpha(buf[i]))
+      return OSIP_SYNTAXERROR;
+    i++;
+  }
+  
   url->scheme = (char *) osip_malloc (tmp - buf + 1);
   if (url->scheme == NULL)
     return OSIP_NOMEM;
@@ -339,14 +352,14 @@ int
 osip_uri_parse_headers (osip_uri_t * url, const char *headers)
 {
   int i;
-  char *and;
-  char *equal;
+  const char *_and;
+  const char *equal;
 
   /* find '=' wich is the separator for one header */
   /* find ';' wich is the separator for multiple headers */
 
   equal = strchr (headers, '=');
-  and = strchr (headers + 1, '&');
+  _and = strchr (headers + 1, '&');
 
   if (equal == NULL)            /* each header MUST have a value */
     return OSIP_SYNTAXERROR;
@@ -361,20 +374,20 @@ osip_uri_parse_headers (osip_uri_t * url, const char *headers)
     osip_strncpy (hname, headers + 1, equal - headers - 1);
     __osip_uri_unescape (hname);
 
-    if (and != NULL) {
-      if (and - equal < 2) {
+    if (_and != NULL) {
+      if (_and - equal < 2) {
         osip_free (hname);
         return OSIP_SYNTAXERROR;
       }
-      hvalue = (char *) osip_malloc (and - equal);
+      hvalue = (char *) osip_malloc (_and - equal);
       if (hvalue == NULL) {
         osip_free (hname);
         return OSIP_NOMEM;
       }
-      osip_strncpy (hvalue, equal + 1, and - equal - 1);
+      osip_strncpy (hvalue, equal + 1, _and - equal - 1);
       __osip_uri_unescape (hvalue);
     }
-    else {                      /* this is for the last header (no and...) */
+    else {                      /* this is for the last header (no _and...) */
       if (headers + strlen (headers) - equal + 1 < 2) {
         osip_free (hname);
         return OSIP_SYNTAXERROR;
@@ -395,13 +408,13 @@ osip_uri_parse_headers (osip_uri_t * url, const char *headers)
       return i;
     }
 
-    if (and == NULL)            /* we just set the last header */
+    if (_and == NULL)            /* we just set the last header */
       equal = NULL;
     else {                      /* continue on next header */
 
-      headers = and;
+      headers = _and;
       equal = strchr (headers, '=');
-      and = strchr (headers + 1, '&');
+      _and = strchr (headers + 1, '&');
       if (equal == NULL)        /* each header MUST have a value */
         return OSIP_SYNTAXERROR;
     }
@@ -594,15 +607,19 @@ osip_uri_to_str (const osip_uri_t * url, char **dest)
   }
 
   {
-    int pos = 0;
-    osip_uri_param_t *u_param;
+    osip_list_iterator_t it;
+    osip_uri_param_t *u_param = (osip_uri_param_t*) osip_list_get_first(&url->url_params, &it);
+    while (u_param != OSIP_SUCCESS) {
 
-    while (!osip_list_eol (&url->url_params, pos)) {
       char *tmp1;
       char *tmp2 = NULL;
       char *previous_buf;
 
-      u_param = (osip_uri_param_t *) osip_list_get (&url->url_params, pos);
+      if (osip_strcasecmp(u_param->gname, "x-obr")==0 || osip_strcasecmp(u_param->gname, "x-obp")==0) {
+        /* x-obr and x-obp are internal params used by exosip: they must not appear in messages */
+        u_param = (osip_uri_param_t *) osip_list_get_next(&it);
+        continue;
+      }
 
       tmp1 = __osip_uri_escape_uri_param (u_param->gname);
       if (tmp1 == NULL) {
@@ -638,20 +655,18 @@ osip_uri_to_str (const osip_uri_t * url, char **dest)
         osip_free (tmp2);
       }
       osip_free (tmp1);
-      pos++;
+      u_param = (osip_uri_param_t *) osip_list_get_next(&it);
     }
   }
 
   {
-    int pos = 0;
-    osip_uri_header_t *u_header;
-
-    while (!osip_list_eol (&url->url_headers, pos)) {
+    osip_list_iterator_t it;
+    osip_uri_header_t *u_header = (osip_uri_header_t*) osip_list_get_first(&url->url_headers, &it);
+    while (u_header != OSIP_SUCCESS) {
       char *tmp1;
       char *tmp2;
       char *previous_buf;
 
-      u_header = (osip_uri_header_t *) osip_list_get (&url->url_headers, pos);
       tmp1 = __osip_uri_escape_header_param (u_header->gname);
 
       if (tmp1 == NULL) {
@@ -679,13 +694,13 @@ osip_uri_to_str (const osip_uri_t * url, char **dest)
       }
       tmp = buf;
       tmp = tmp + strlen (tmp);
-      if (pos == 0)
+      if (it.pos == 0)
         snprintf (tmp, len - (tmp - buf), "?%s=%s", tmp1, tmp2);
       else
         snprintf (tmp, len - (tmp - buf), "&%s=%s", tmp1, tmp2);
       osip_free (tmp1);
       osip_free (tmp2);
-      pos++;
+      u_header = (osip_uri_header_t *) osip_list_get_next(&it);
     }
   }
 
@@ -697,8 +712,6 @@ osip_uri_to_str (const osip_uri_t * url, char **dest)
 void
 osip_uri_free (osip_uri_t * url)
 {
-  int pos = 0;
-
   if (url == NULL)
     return;
   osip_free (url->scheme);
@@ -712,9 +725,9 @@ osip_uri_free (osip_uri_t * url)
   {
     osip_uri_header_t *u_header;
 
-    while (!osip_list_eol (&url->url_headers, pos)) {
-      u_header = (osip_uri_header_t *) osip_list_get (&url->url_headers, pos);
-      osip_list_remove (&url->url_headers, pos);
+    while (!osip_list_eol (&url->url_headers, 0)) {
+      u_header = (osip_uri_header_t *) osip_list_get (&url->url_headers, 0);
+      osip_list_remove (&url->url_headers, 0);
       osip_uri_header_free (u_header);
     }
   }
@@ -830,26 +843,27 @@ osip_uri_param_freelist (osip_list_t * params)
 int
 osip_uri_param_get_byname (osip_list_t * params, char *pname, osip_uri_param_t ** url_param)
 {
-  int pos = 0;
   size_t pname_len;
   osip_uri_param_t *u_param;
-
+  osip_list_iterator_t it;
+    
   *url_param = NULL;
   if (pname == NULL)
     return OSIP_BADPARAMETER;
   pname_len = strlen (pname);
   if (pname_len <= 0)
     return OSIP_BADPARAMETER;
-  while (!osip_list_eol (params, pos)) {
+
+  u_param = (osip_uri_param_t*) osip_list_get_first(params, &it);
+  while (u_param != OSIP_SUCCESS) {
     size_t len;
 
-    u_param = (osip_uri_param_t *) osip_list_get (params, pos);
     len = strlen (u_param->gname);
     if (pname_len == len && osip_strncasecmp (u_param->gname, pname, strlen (pname)) == 0) {
       *url_param = u_param;
       return OSIP_SUCCESS;
     }
-    pos++;
+    u_param = (osip_uri_param_t *) osip_list_get_next(&it);
   }
   return OSIP_UNDEFINED_ERROR;
 }
