@@ -1,6 +1,6 @@
 /*
   The oSIP library implements the Session Initiation Protocol (SIP -rfc3261-)
-  Copyright (C) 2001-2012 Aymeric MOIZARD amoizard@antisip.com
+  Copyright (C) 2001-2015 Aymeric MOIZARD amoizard@antisip.com
   
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
@@ -251,6 +251,8 @@ osip_via_parse (osip_via_t * via, const char *hvalue)
     if (via->port == NULL)
       return OSIP_NOMEM;
     osip_clrncpy (via->port, port + 1, via_params - port - 1);
+    if(osip_atoi(via->port) < 0)
+      return OSIP_SYNTAXERROR;
   }
   else
     port = via_params;
@@ -311,12 +313,9 @@ osip_via_to_str (const osip_via_t * via, char **dest)
 
 
   {
-    int pos = 0;
-    osip_generic_param_t *u_param;
-
-    while (!osip_list_eol (&via->via_params, pos)) {
-      u_param = (osip_generic_param_t *) osip_list_get (&via->via_params, pos);
-
+    osip_list_iterator_t it;
+    osip_generic_param_t *u_param = (osip_generic_param_t*) osip_list_get_first(&via->via_params, &it);
+    while (u_param != OSIP_SUCCESS) {
       if (u_param->gvalue == NULL)
         plen = strlen (u_param->gname) + 2;
       else
@@ -329,7 +328,7 @@ osip_via_to_str (const osip_via_t * via, char **dest)
         sprintf (tmp, ";%s", u_param->gname);
       else
         sprintf (tmp, ";%s=%s", u_param->gname, u_param->gvalue);
-      pos++;
+      u_param = (osip_generic_param_t *) osip_list_get_next(&it);
     }
   }
 
