@@ -1,6 +1,6 @@
 /*
   The oSIP library implements the Session Initiation Protocol (SIP -rfc3261-)
-  Copyright (C) 2001-2012 Aymeric MOIZARD amoizard@antisip.com
+  Copyright (C) 2001-2015 Aymeric MOIZARD amoizard@antisip.com
   
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
@@ -325,12 +325,11 @@ __osip_sdp_append_string (char **string, int *size, char *cur, char *string_osip
 {
   int length = (int) strlen (string_osip_to_append);
 
-  if (cur - (*string) + length > *size) {
+  if (cur - (*string) + length +1 > *size) {
     int length2;
-
     length2 = (int) (cur - *string);
-    (*string) = osip_realloc ((*string), *size + length + 10);
-    *size = *size + length + 10;
+    (*string) = osip_realloc ((*string), *size + length + 500);
+    *size = *size + length + 500; /* optimize: avoid too much realloc */
     cur = (*string) + length2;  /* the initial allocation may have changed! */
   }
   osip_strncpy (cur, string_osip_to_append, length);
