@@ -44,8 +44,8 @@ main (int argc, char **argv)
     exit (0);
   }
 
-  a_contact = (char *) osip_malloc (200);
-  res = fgets (a_contact, 200, contacts_file);  /* lines are under 200 */
+  a_contact = (char *) osip_malloc (500);
+  res = fgets (a_contact, 500, contacts_file);  /* lines are under 200 */
   while (res != NULL) {
 
     int errcode;
@@ -62,6 +62,21 @@ main (int argc, char **argv)
       if (errcode != -1) {
         if (osip_contact_to_str (contact, &dest) != -1) {
           printf ("result:           |%s|\n", dest);
+
+	  {
+	    int pos = 0;
+	    osip_generic_param_t *u_param;
+	    while (!osip_list_eol (&contact->gen_params, pos)) {
+	      u_param = (osip_generic_param_t *) osip_list_get (&contact->gen_params, pos);
+	      
+	      if (u_param->gvalue == NULL)
+		printf ("result:       ;%s\n", u_param->gname);
+	      else
+		printf ("result:       ;%s=%s\n", u_param->gname, u_param->gvalue);
+	      pos++;
+	    }
+	  }
+
           osip_free (dest);
         }
       }
@@ -70,7 +85,7 @@ main (int argc, char **argv)
       osip_contact_free (contact);
       printf ("=================================================\n");
     }
-    res = fgets (a_contact, 200, contacts_file);        /* lines are under 200 */
+    res = fgets (a_contact, 500, contacts_file);        /* lines are under 200 */
   }
   osip_free (a_contact);
   return 0;
